#include <iostream>
#include <iomanip>
#include <fstream>
#include <math.h>
#include <stdio.h>
#include <cstdlib>
#include "cell.h"
using namespace std;

int main()
{
	//********************** Information on Hydroshed original data. Should not change ******************
	/* HydroSHED data for Asia, Flow Direction and DEM
	LL corner Lat =-10
	LL corner Lon =55
	UR corner Lat =60
	UR corner Lon =180	
	*/
	const int HyRow=8400;
	const int HyCol=15000;
	double  Dx = 0.00833333; // cellsize in degrees
	double  ULLat = 60-Dx/2; // The latitude of the center of Upper Left (North West) grid. 
	                         // 60 (degree) is the latitude of North West corner of Asia data of HydroSHED.
	double  ULLon = 55+Dx/2; // The longitude of the center of Upper Left (North West) grid. 
	                         // 55 (degree) is the longitude of North West corner of Asia data of HydroSHED.
	//***************************************************************************************************

	short int dem, dir;
	char filename[256];
	float nodata =-9999;
	int nodata4bin=-9999;
	// ##############CHANGES  ONLY BELOW ACCORDING TO THE AREA INTERESTED IN################################
	// define the calculation region //  one grid is =1/120 degrees,
	///*Chaopria thailand */
	//const int Row = 120*8;		// No of row = degrees in latitude direction * 120
	//const int Col = 120*6;		// No of column = degrees in Longitude direction * 120 
	//const int NWLAT = 21;			// Upper left corner Latitude (deg.)
	//const int NWLON = 97;			// Left corner longitude (deg.)
	const char *projectName = "";   // 
	const int forOneCell = 1;	/*set always one */
	// #####################################################################################################

	//read study area for cliping topographic data.
	cout << "1) Reading location of study area." << endl;
	ifstream finClipArea("./input/clipArea.dat");
	if(!finClipArea){
		cout << "clipArea.dat cannot open." << endl;
		return 1;
	}

	int wRow, wCol, wNWLAT, wNWLON;

	finClipArea >> wRow; cout << "  Number of row : " << wRow << endl;
	finClipArea >> wCol; cout << "  Number of column : " << wCol << endl;
	finClipArea >> wNWLAT; cout << "  Latitude (degree) of North West corner : " << wNWLAT << endl;
	finClipArea >> wNWLON; cout << "  Longitude (degree) of North West corner : " << wNWLON << endl;

	const int Row = wRow;
	const int Col = wCol;
	const int NWLAT = wNWLAT;
	const int NWLON = wNWLON;

	finClipArea.close();


	/*--------------------------------File Names-------------------------------------------------*/	
	FILE *HyDem;
	FILE *HyDir;
	FILE *ClipHyDem;
	FILE *ClipHyDir;
	errno_t error;
	int cut_i, cut_j;   // Upper left corner i and  j of the clipped box

	//make output project directory
	//sprintf(filename,"mkdir ./output/%s",projectName);
	//system (filename);
	
	//memory allocation
	class Hydroshed** HyshedObject;

	HyshedObject = new class Hydroshed*[HyRow];
	for(int i=0; i < HyRow; i++){
		
		HyshedObject[i] = new class Hydroshed[HyCol];
	}

	//read topographic data generated from HydroSHED
	if(error = fopen_s(&HyDem, "./input/as_dem_30s_clipped_55--10-180-60.bin", "rb") != NULL) {
		cout << "Hydroshed DEM cannot open." << endl;  //hunu
		exit(1);
	} 

	if(error = fopen_s(&HyDir, "./input/as_dir_30s_clipped_55--10-180-60.bin", "rb") != NULL) {
		cout << "Hydroshed Flow direction file cannot open." << endl;  //hunu
		exit(1);
	} 
	
	// read the Hydroshed data and assigned to an array
	cout << "2) Reading topography data." << endl;
	for (int i=0; i<HyRow; i++){
		if (i%100 == 0) cout << "  reading " << i << "/" << HyRow<< "th row data.\r";
		for (int j=0; j<HyCol; j++){
			fread(&dem, sizeof(short int), 1, HyDem);	
			fread(&dir, sizeof(short int), 1, HyDir);
			
			HyshedObject[i][j].HySetvalues(dem,dir);
			
			//if (HyshedObject[i][j].dem >0){
			//cout << HyshedObject[i][j].dem << HyshedObject[i][j].dir << endl;
			//}
		}
	}	
	fclose(HyDem);
	fclose(HyDir);

	//clipping
	cout << "\n3) Cripping topography data." << endl;
	double tempLat, tempLon;
	tempLat=NWLAT-Dx/2;
	tempLon=NWLON+Dx/2;
	
	cut_i= (int) ((ULLat-tempLat)/Dx +0.5);
	cut_j= (int)((tempLon-ULLon)/Dx +0.5);
	
	// cout << cut_i <<" " <<cut_j <<" "<<ULLat  <<endl;
	
	//write data	
	sprintf_s(filename, 256, "./output/DemN.bin", projectName);
	if(error = fopen_s(&ClipHyDem, filename, "wb") != NULL) {
		cout << "DemN.bin file cannot open." << endl;  //hunu
		exit(1);
	} 
	for (int i=cut_i; i<cut_i+Row; i++){
		for (int j=cut_j; j<cut_j+Col; j++){
			fwrite(&HyshedObject[i][j].dem, sizeof(short int), 1, ClipHyDem);  // write 2 bite integer value
		}
	}		
	cout << "  Writing " << filename << " done." <<endl;
	fclose(ClipHyDem);

	sprintf_s(filename, 256, "./output/DownN.bin", projectName);
	if(error = fopen_s(&ClipHyDir, filename, "wb") != NULL) {
		cout << "DownN.bin file cannot open." << endl;  //hunu
		exit(1);
	} 
	for (int i=cut_i; i<cut_i+Row; i++){
		for (int j=cut_j; j<cut_j+Col; j++){
			fwrite(&HyshedObject[i][j].dir, sizeof(short int), 1, ClipHyDir);  // write 2 bite integer value		
		}
	}		
	cout << "  Writing " << filename << " done." <<endl;
	fclose(ClipHyDir);
	
	//write the clipped data into ESRI ASCII file format
	sprintf_s(filename, 256, "./output/ESRIcutdem.asc", projectName);
	ofstream foutDem(filename);
	if(!foutDem){
		cout << "cutdem file cannot open \n";
		return 1;
	}
	//ESRI header
	foutDem << "ncols "<<Col<<"\n";
	foutDem << "nrows "<<Row<<"\n";
	foutDem << "xllcorner "<<NWLON<<"\n";
	foutDem << "yllcorner "<<NWLAT-(Row/120.0)<<"\n";
	foutDem << "cellsize "<<Dx<<"\n";
	foutDem << "NODATA_value "<<nodata<< "\n";

	for(int i=cut_i;i<cut_i+Row;i++){
		for(int j=cut_j;j<cut_j+Col;j++){
			foutDem << " " << HyshedObject[i][j].dem;
		}
		foutDem <<"\n";
	}
	cout << "  Writing " << filename << " done." <<endl;
	foutDem.close();

	
	sprintf_s(filename, 256, "./output/ESRIcutdir.asc", projectName);	
	ofstream foutDir(filename);
	if(!foutDir){
		cout << "ESRIcutdir.asc file cannot open \n";
		return 1;
	}
	//ESRI header
	foutDir << "ncols "<<Col<<"\n";
	foutDir << "nrows "<<Row<<"\n";
	foutDir << "xllcorner "<<NWLON<<"\n";
	foutDir << "yllcorner "<<NWLAT-(Row/120.0)<<"\n";
	foutDir << "cellsize "<<Dx<<"\n";
	foutDir << "NODATA_value "<<nodata<< "\n";

	for(int i=cut_i;i<cut_i+Row;i++){
		for(int j=cut_j;j<cut_j+Col;j++){
			foutDir << " " << HyshedObject[i][j].dir;
		}
		foutDir <<"\n";
	}

	cout << "  Writing " << filename << " done." <<endl;
	foutDir.close();
	
	delete[] HyshedObject;
	//clipping finished.


    // data processing.
	cout << "4) Processing topography data." << endl;
	FILE *fMed;	
	FILE *fCorrectedMed;
	//FILE *fDistance;
	FILE *fDown;
	FILE *fBasin;
	FILE *fRiverNumber;
	FILE *fDown2; //hunu
	FILE *ffacc;
	short int  med, down;  //hunu
	int enterKey;
	float FaccGrad;

	/*--------------------------------Memory allocation---------------------------------*/
	class Direction** cellObject;

	cout << "  Memory allocation." << endl;
	cellObject = new class Direction*[Row/forOneCell];
	for(int i=0; i<Row/forOneCell; i++){
		cellObject[i] = new class Direction[Col/forOneCell];
	}

	/*---------------------------------------------------------------------*/
	// cout << "latitude  longitude  longitudinalLength  med \n";

	sprintf_s(filename, 256, "./output/DemN.bin", projectName);
	if(error = fopen_s(&fMed, filename, "rb") != NULL) {
		cout << "fMed.bin file cannot open." << endl;  //hunu
		exit(1);
	} 

	sprintf_s(filename, 256, "./output/DownN.bin", projectName);
	if(error = fopen_s(&fDown2, filename, "rb") != NULL) {
		cout << "DownN.bin file cannot open." << endl;  //hunu
		exit(1);
	} 

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			fread(&med, sizeof(short int), 1, fMed);
			fread(&down, sizeof(short int), 1, fDown2); //hunu
			cellObject[i][j].setElevation(i, j, NWLAT, NWLON, forOneCell,  med,Row/forOneCell,Col/forOneCell,down); //hunu
		}
	}

	fclose(fMed);
	fclose(fDown2); //hunu
	
	//This part (below) is to make data even the area is not surrounded by the sea	
	for(int i=0; i<Row/forOneCell; i++){ 
		for(int j=0; j<Col/forOneCell; j++){
			if (i==0 || j==0 || i== (Row-1) || j==(Col-1)){
				if (cellObject[i][j].down ==0){
					cellObject[i][j].down=0;
					//cout << i <<"-"<<j<<"\n";
				}
				else {
					cellObject[i][j].down=0;
					//cout << i <<"-"<<j<<"\n";
				}
				//cellObject[i][j].med=-9999;
				//cout << cellObject[i][j].down <<"\n";
			}
		}
	}
	//This part (above) is to make data even the area is not surrounded by the sea
	
	//hunu HydroSHED flow direction sign convention is diffrent from the convention used here 
	// convert Hydroshed direction

	cout << "  Assigning elevation data." << endl;

	for(int i=0; i<Row/forOneCell; i++){ 
		for(int j=0; j<Col/forOneCell; j++){

			if (cellObject[i][j].down ==1){
				cellObject[i][j].down=3;
				//cout << cellObject[i][j].down <<"\n";
			}
			else if (cellObject[i][j].down ==2){
				cellObject[i][j].down=4;
			}		
			else if (cellObject[i][j].down ==4){
				cellObject[i][j].down=5;
			}		
			else if (cellObject[i][j].down ==8){
				cellObject[i][j].down=6;
			}	
			else if (cellObject[i][j].down ==16){
				cellObject[i][j].down=7;
			}	
			else if (cellObject[i][j].down ==32){
				cellObject[i][j].down=8;
			}	
			else if (cellObject[i][j].down ==64){
				cellObject[i][j].down=1;
			}	
			else if (cellObject[i][j].down ==128){
				cellObject[i][j].down=2;
			}
			else if (cellObject[i][j].down ==0){
				cellObject[i][j].down=0;
			}				
			else if (cellObject[i][j].down ==-1){
				// cellObject[i][j].med=-9999; 
				// According to HydroShed, this is a inland sink or pit, according to this programme 
				//cout << i <<" "<<j<<" " << cellObject[i][j].med<<endl;
				// we set this as a cell in the sea.
				// look at the function mouthAndHollow()   				
				//cout << cellObject[i][j].down <<"\n";
			}	
			else if (cellObject[i][j].down ==-9999){
				cellObject[i][j].down=-1;
				//cout << i <<"-"<<j << "cellObject.down is <0\n";
			}				
			else {
				cout << i << "-" << j << "-" << down << "cellObject.down is wrong.\n";
			}
			//if (i==203 && j==43 || i==203 && j==44 || i==203 && j==45 ||i==202 && j==43 || i==202 && j==44 || i==202 && j==45 ||i==204 && j==43 || i==204 && j==44 || i==204 && j==45  ){
			//	cout << i << ", " << j << ", "<<cellObject[i][j].med << ", "<< cellObject[i][j].down<<endl;
			//cout << i << ", " << j << ", "<<cellObject[i][j].basin << ", "<< cellObject[i][j].riverNumber<<endl;
			//}
		}
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			cellObject[i][j].setNULL();		//
			if(cellObject[i][j].med==-9999.0)
			continue;

			//		cellObject[i][j].setNeighbor(cellObject, Row/forOneCell, Col/forOneCell, forOneCell);

			cellObject[i][j].setNeighbor(cellObject, i, j);
		}	
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].med==-9999.0)
			continue;

			if (cellObject[i][j].down==-1){
				cellObject[i][j].SinkSet();
			}
		}	
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if(cellObject[i][j].med == -9999.0){
				cellObject[i][j].down = -1;
				continue; 
			}
			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell 
				|| cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell)
			continue;
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell 
				|| cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell)
			continue;
			cellObject[i][j].inlandOrCoastal();	
		}
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].med == -9999.0)
			continue;
			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell 
				|| cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell)
			continue;
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell 
				|| cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell)
			continue;
			
			cellObject[i][j].compareSlopeOfMediumElevation(1);
			cellObject[i][j].mouthAndHollow();
		}
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].med == -9999.0)
			continue;

			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell 
				|| cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell)
			continue;
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell 
				|| cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell)
			continue;
			
			if(cellObject[i][j].down !=0 ){

				cellObject[i][j].setDownStreamGrid();
			}
		}
	}

	/*--------------------------------Set Upper grid-----------------------------------------------*/
	cout << "  Setting upper grid number.\n";

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			//excluding the cells which dose not need to set upper cell
			/* sea */
			if(cellObject[i][j].med == -9999.0){
				cellObject[i][j].sizeOfBasin = -9999; //originally this was 0;
				continue;
			}

			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell 
				|| cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell) {
					continue;
			}
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell
				|| cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell) {
					continue;
			}
			if(cellObject[i][j].downStreamGrid==NULL) {
				continue;
			}
			cellObject[i][j].setUpStreamGrid();
			//cout << i<<"-"<<j<<"this is 5 \n";
		}
	}

	// hunu In HydroSHED flow direction map, all grids at the sea is set to "0".
    // but in this programme "0" should set only for the outlet of a basin.

	cout << "  Setting the outlet.\n";
	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if(cellObject[i][j].down==0){
				cellObject[i][j].setDownAtMouth(); 
			}
		}
	}

	
	/*--------------------------------Calculate the distance from the mouth---------------------------------------------*/
	cout << "  Calculating the distance from the outlet.\n";
	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			
			if(cellObject[i][j].down==0){
				cellObject[i][j].distanceFromMouth=0;
			}
		}
	}

	int distanceNum=0;
	int numberOfNotSettingTime=0;

	while(numberOfNotSettingTime < Row/forOneCell*Col/forOneCell){

		numberOfNotSettingTime=0;

		for(int i=0; i<Row/forOneCell; i++){
			for(int j=0; j<Col/forOneCell; j++){
				if(cellObject[i][j].distanceFromMouth==distanceNum){
					cellObject[i][j].setDistanceFromMouth();
				}
				else{
					numberOfNotSettingTime++;
				}
			}
		}
		distanceNum++;
	}
	
	/*--------------------------------set basin size---------------------------------------------------*/
	cout << "  Calculating basin size.\n";
	for(int k=distanceNum; k>-1; k--){
		for(int i=0; i<Row/forOneCell; i++){
			for(int j=0; j<Col/forOneCell; j++){
				if(cellObject[i][j].distanceFromMouth==k){
					cellObject[i][j].setSizeOfBasinEx(); 
				}
			}
		}
	}

	/*--------------------------------riverNumber---------------------------------------------------*/
	cout << "  Setting river number.\n";

	int rNum;
	rNum = 1;

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].down == 0){
				cellObject[i][j].riverNumber = rNum;
				rNum++;
			}
		}
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			
			if(cellObject[i][j].sizeOfBasin == 1){
				//cout << i <<"-"<< j<<" this is 7 \n";
				cellObject[i][j].numberRiverNumber();
			}
		}
	}


	/*--------------------------------writing output files---------------------------------------------*/
	cout <<"5) Writing output files" << endl;

	sprintf_s(filename, 256, "./output/correctedMed.bin", projectName);
	if(error = fopen_s(&fCorrectedMed, filename, "wb") != NULL) {
		cout << "correctedMed.bin file cannot open." << endl;
		exit(1);
	} 

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			fwrite(&cellObject[i][j].med, sizeof(double), 1, fCorrectedMed);  // write 8 bite real value
		}
	}
	cout << "  Writing " << filename <<endl;
	fclose(fCorrectedMed);
	
	/*--------------------------------DEM--------------------------------------------*/
	enterKey = NWLON*3600 + forOneCell*30*1/2;
	
	double xllcorner, yllcorner;
	xllcorner=NWLON;
	yllcorner=NWLAT-(Row/120.0);
		
	sprintf_s(filename, 256, "./output/ESRIDEMMed.asc", projectName);
	ofstream fout3(filename);
	if(!fout3){
		cout << "ESRIDEMMed.txt file can not open\n";
		return 1;
	}
	//ESRI header

	fout3 << "ncols "<<Col<<"\n";
	fout3 << "nrows "<<Row<<"\n";
	fout3 << "xllcorner "<<xllcorner<<"\n";
	fout3 << "yllcorner "<<yllcorner<<"\n";
	fout3 << "cellsize "<<Dx<<"\n";
	fout3 << "NODATA_value "<<nodata;

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if(cellObject[i][j].longitude == enterKey){
				fout3 << "\n";
			}

			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell || cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell){
			fout3 << " " << nodata;
				continue;
			}
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell || cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell){
			fout3 << " " << nodata;
				continue;
			}

			fout3 << " " << cellObject[i][j].med;
				
			/*if(i%8 == 0){
				fout3 << "\n";
			}
			fout3 << " " << cellObject[i][j].med;*/
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fout3.close();

	/*--------------------------------Distance from the sea --------------------------------------------*/
	/*(As this information is not used in the Runoff model this was comment out)
	sprintf_s(filename, 256, "./output/distance.bin", projectName);
	if(error = fopen_s(&fDistance, filename, "wb") != NULL) {
		cout << "distance.bin file cannot open." << endl;
		exit(1);
	} 

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			
			fwrite(&cellObject[i][j].distanceFromMouth, sizeof(int), 1, fDistance);
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fclose(fDistance);

	sprintf_s(filename, 256, "./output/ESRIDistance.asc", projectName);
	ofstream fout2(filename);
	if(!fout2){
		cout << "ESRIDistance.txt file can not open\n";
		return 1;
	}
	//ESRI header
	fout2 << "ncols "<<Col<<"\n";
	fout2 << "nrows "<<Row<<"\n";
	fout2 << "xllcorner "<<xllcorner<<"\n";
	fout2 << "yllcorner "<<yllcorner<<"\n";
	fout2 << "cellsize "<<Dx<<"\n";
	fout2 << "NODATA_value "<<-1;

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].longitude == enterKey){
				fout2 << "\n";
			}

			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell || cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell){
			fout2 << " " << -1;
				continue;
			}
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell || cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell){
			fout2 << " " << -1;
				continue;
			}


			fout2 << " " << cellObject[i][j].distanceFromMouth;
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fout2.close();
(As this information is not used in the Runoff model this was comment out)*/


	/*--------------------------------Flow direction file---------------------------------------------*/

	sprintf_s(filename, 256, "./output/down.bin", projectName);
	if(error = fopen_s(&fDown, filename, "wb") != NULL) {
		cout << "down.bin file cannot open." << endl;
		exit(1);
	} 
	
	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell 
				|| cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell){
				fwrite(&nodata4bin, sizeof(int), 1, fDown); // write 4 bite integer value
				continue;
			}
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell 
				|| cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell){
				fwrite(&nodata4bin, sizeof(int), 1, fDown); // write 4 bite integer value
				continue;
			}
			fwrite(&cellObject[i][j].down, sizeof(int), 1, fDown); // write 4 bite integer value
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fclose(fDown);

	sprintf_s(filename, 256, "./output/ESRIFlowDIR.asc", projectName);
	ofstream fout(filename);
	if(!fout){
		cout << "file can not open\n";
		return 1;
	}
	//ESRI header
	fout << "ncols "<<Col<<"\n";
	fout << "nrows "<<Row<<"\n";
	fout << "xllcorner "<<xllcorner<<"\n";
	fout << "yllcorner "<<yllcorner<<"\n";
	fout << "cellsize "<<Dx<<"\n";
	fout << "NODATA_value "<<nodata;

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			
			if(cellObject[i][j].longitude == enterKey){
				fout << "\n";
			}
			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell 
				|| cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell){
				fout << " " << nodata;
				continue;
			}
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell 
				|| cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell){
				fout << " " << nodata;
				continue;
			}


			// change the Flow direction to ESRI arc grid format

			if (cellObject[i][j].down ==3){
				cellObject[i][j].down=1;
				//cout << cellObject[i][j].down <<"\n";
			}
			else if (cellObject[i][j].down ==4){
				cellObject[i][j].down=2;
			}		
			else if (cellObject[i][j].down ==5){
				cellObject[i][j].down=4;
			}		
			else if (cellObject[i][j].down ==6){
				cellObject[i][j].down=8;
			}	
			else if (cellObject[i][j].down ==7){
				cellObject[i][j].down=16;
			}	
			else if (cellObject[i][j].down ==8){
				cellObject[i][j].down=32;
			}	
			else if (cellObject[i][j].down ==1){
				cellObject[i][j].down=64;
			}	
			else if (cellObject[i][j].down ==2){
				cellObject[i][j].down=128;
			}
			else if (cellObject[i][j].down ==0){
				cellObject[i][j].down=0;
			}				
			//else if (cellObject[i][j].down ==-1){
			//	cellObject[i][j].down=-9999;
			//	cout << cellObject[i][j].down <<"\n";
			//cout << i <<"-"<<j << "cellObject.down is <0\n";
			//}	
			else if (cellObject[i][j].down ==-1){ 
				cellObject[i][j].down=-9999;  
				//cout << i <<"-"<<j << "cellObject.down is <0\n";
			}				
			else {
				cout << i <<"-"<<j << "-"<< down << "cellObject.down is wrong\n";
			}
			//----------------------------------------------------------------------------------					
			
			fout << " " << cellObject[i][j].down;
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fout.close();
	
	// Flow accumulation file to visualize on GRADS ###################################################
	
	sprintf_s(filename, 256, "./output/FaccGrads.bin", projectName);
	if(error = fopen_s(&ffacc, filename, "wb") != NULL) {
		cout << "faccgrads.bin file cannot open." << endl;
		exit(1);
	} 

	for(int i=Row/forOneCell-1; i>=0; i--){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell
				|| cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell){
				fwrite(&nodata, sizeof(float), 1, ffacc); // write 4 bite real value
				continue;
			}
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell
				|| cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell){
				fwrite(&nodata, sizeof(float), 1, ffacc); // write 4 bite real value
				continue;
			}
            FaccGrad = (float)cellObject[i][j].sizeOfBasin;
			fwrite(&FaccGrad, sizeof(float), 1, ffacc); // write 4 bite real value
			// cout << FaccGrad << endl;
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fclose(ffacc);
	// Above file is the Flow accumulation file to visualize  on GRADS ###################################################

	/*--------------------------------this is kind of a flow accumulation file--------------------------------------------*/
	sprintf_s(filename, 256, "./output/basin.bin", projectName);
	if(error = fopen_s(&fBasin, filename, "wb") != NULL) {
		cout << "basin.bin file cannot open." << endl;
		exit(1);
	} 

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			
			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell 
				|| cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell){
				fwrite(&nodata4bin, sizeof(int), 1, fBasin); // write 4 bite integer value
				continue;
			}
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell 
				|| cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell){
				fwrite(&nodata4bin, sizeof(int), 1, fBasin); // write 4 bite integer value
				continue;
			}

			fwrite(&cellObject[i][j].sizeOfBasin, sizeof(int), 1, fBasin); // write 4 bite integer value
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fclose(fBasin);

	sprintf_s(filename, 256, "./output/ESRIBasin.asc", projectName);
	ofstream fout1(filename);
	if(!fout1){
		cout << "ESRIBasin.txt file can not open\n";
		return 1;
	}
	fout1 << "ncols "<<Col<<"\n";
	fout1 << "nrows "<<Row<<"\n";
	fout1 << "xllcorner "<<xllcorner<<"\n";
	fout1 << "yllcorner "<<yllcorner<<"\n";
	fout1 << "cellsize "<<Dx<<"\n";
	fout1 << "NODATA_value "<<nodata;

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if(cellObject[i][j].longitude == enterKey){
				fout1 << "\n";
			}


			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell || cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell){
			fout1 << " " << nodata;
			continue;
			}

			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell || cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell){
			fout1 << " " << nodata;
				continue;
			}


			fout1 << " " << cellObject[i][j].sizeOfBasin;
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fout1.close();

	/*--------------------------------riverNumber---------------------------------------------*/
	sprintf_s(filename, 256, "./output/riverNumber.bin", projectName);
	if(error = fopen_s(&fRiverNumber, filename, "wb") != NULL) {
		cout << "riverNumber.bin file cannot open." << endl;
		exit(1);
	} 

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell 
				|| cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell){

				fwrite(&nodata4bin, sizeof(int), 1, fRiverNumber); // write 4 bite integer value
				continue;
			}
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell 
				|| cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell){

				fwrite(&nodata4bin, sizeof(int), 1, fRiverNumber); // write 4 bite integer value
				continue;
			}

			fwrite(&cellObject[i][j].riverNumber, sizeof(int), 1, fRiverNumber); // write 4 bite integer value
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fclose(fRiverNumber);

	sprintf_s(filename, 256, "./output/ESRIRiverNumber.asc", projectName);
	ofstream fout4(filename);
	if(!fout4){
		cout << "ESRIRiverNumber.asc riverNumber file can not open\n";
		return 1;
	}
	fout4 << "ncols "<<Col<<"\n";
	fout4 << "nrows "<<Row<<"\n";
	fout4 << "xllcorner "<<xllcorner<<"\n";
	fout4 << "yllcorner "<<yllcorner<<"\n";
	fout4 << "cellsize "<<Dx<<"\n";
	fout4 << "NODATA_value "<<-1 ;
	
	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){

			if(cellObject[i][j].longitude == enterKey){
				fout4 << "\n";
			}

			if(cellObject[i][j].latitude==NWLAT*3600-15*forOneCell 
				|| cellObject[i][j].latitude==NWLAT*3600-30*forOneCell*(Row/forOneCell-1)-15*forOneCell){
			fout4 << " " << -1;
				continue;
			}
			if(cellObject[i][j].longitude==NWLON*3600+15*forOneCell 
				|| cellObject[i][j].longitude==NWLON*3600+30*forOneCell*(Col/forOneCell-1)+15*forOneCell){
			fout4 << " " << -1;
				continue;
			}


			//fout4 << setw(5) << cellObject[i][j].riverNumber;
			fout4 << " " << cellObject[i][j].riverNumber;
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fout4.close();

	//--- riverNumber list ŉƂȂObh̏o
	sprintf_s(filename, 256, "./output/riverNumList.txt", projectName);
	ofstream fRiverNumList(filename);
	if(!fRiverNumList){
		cout << "riverNumList.txt file can not open\n";
		return 1;
	}

	for(int i=0; i<Row/forOneCell; i++){
		for(int j=0; j<Col/forOneCell; j++){
			if(cellObject[i][j].down==0){
				fRiverNumList << cellObject[i][j].riverNumber 
					<< " " << i 
					<< " " << j 
					<< " " << cellObject[i][j].longitude
					<< " " << cellObject[i][j].latitude
					<< " " << cellObject[i][j].sizeOfBasin
					<< endl;
			}
		}
	}
	cout << "  Writing " << filename << " done." <<endl;
	fRiverNumList.close();

	delete[] cellObject;

	return 0;
}

// functions  for clipping programme
void Hydroshed::HySetvalues(short int dem1, short int dir1)
{
	dem=dem1;
	dir=dir1;
}



void Direction::compareSlopeOfMediumElevation(int number)
{
	lenOf30SecLat = 926.6;

	SteepestSlope = -50000;
	numberOfDirection = 0;
	
	for(int i=0; i<8; i++){
		if(neighborGrid[i]->med == -9999)
		continue;

		longiudinalDistance = longitudinalLength * abs(longitude - neighborGrid[i]->longitude)/30.0;
		latitudinalDistance = lenOf30SecLat * abs(latitude - neighborGrid[i]->latitude)/30.0;

		distance = sqrt(pow(latitudinalDistance, 2.0) + pow(longiudinalDistance, 2.0));

		Slope[i] = (med - neighborGrid[i]->med)/distance;

		if(SteepestSlope < Slope[i]){
			SteepestSlope = Slope[i];
			// hunu down = i + 1;
		}
	}

	for(int i=0; i<8; i++){
		if(SteepestSlope == Slope[i]){
			Number[i] = number;	
			numberOfDirection++; // if this numberOfDirection >1 , flow from the grid can goes to several (numberOfDirection) grids
		}			
	}
}



void Cell::setElevation(int rowStep, int colStep, int lat, int lon, int numberOfGrid,  int medElevation, int nRow, int nCol, int down1)//hunu
{
	latitude  = lat*3600 - 30*numberOfGrid*rowStep - 15*numberOfGrid;
	longitude = lon*3600 + 30*numberOfGrid*colStep + 15*numberOfGrid;

	longitudinalLength = 926.6*cos(latitude/3600.0/180.0*3.14);
	//if (rowStep >0 && colStep > 0 && rowStep <nRow-1 && colStep< nCol-1){

	med = medElevation;
	
	down = down1; //hunu
	//}
	//else {
	
	//	med = -9999;
	
	//}
}

/*  Initialize flow direction, adjesant grid number and lower grid number */
void Cell::setNULL()
{
	//hunu down=10; 
	distanceFromMouth=-1;
	sizeOfBasin=1;
	riverNumber=-1;

	downStreamGrid=NULL;
	startOFsetSizeOfBasin=false;

	for(int i=0; i<8; i++){
		neighborGrid[i]=NULL;
	}

	for(int i=0; i<7; i++){
		upStreamGrid[i]=NULL;
	}
}


void Cell::setNeighbor(class Direction** cell2, int i, int j)
{	
	neighborGrid[0]=&cell2[i-1][j];
	neighborGrid[1]=&cell2[i-1][j+1];
	neighborGrid[2]=&cell2[i][j+1];
	neighborGrid[3]=&cell2[i+1][j+1];
	neighborGrid[4]=&cell2[i+1][j];
	neighborGrid[5]=&cell2[i+1][j-1];
	neighborGrid[6]=&cell2[i][j-1];
	neighborGrid[7]=&cell2[i-1][j-1];
}

void Cell::SinkSet()
{
	med=-9999;// set the sink as sea
	for(int i=0; i<8; i++){
		neighborGrid[i]->down=0;
	}

}

// hunu In HydroSHED flow direction map, all grids at the sea is set to "0".
// but in this programme "0" should set only for the outlet of a basin.
void Cell::setDownAtMouth()
{
	int hhh=0;
	for(int i=0; i<7; i++){
		if(upStreamGrid[i]!=NULL){
			hhh=1;
		}
	}
	if (hhh==0){
		down=-1;
		med=-9999;
		sizeOfBasin=-1;
		//cout<<"this is 8\n";
	} 
}

void Direction::inlandOrCoastal()
{
	numberOfCoastalGrid=0;

	for(int i=0; i<8; i++){

		Number[i] = 0;
		
		if(neighborGrid[i]->med == -9999)
		numberOfCoastalGrid++;
	}
}

//delete this function
void Direction::reclamationOfHollow(int* g9)
{
	lowElevation = 9999;

	double medDummy;
	medDummy = med;


	for(int i=0; i<8; i++){
		if(lowElevation > neighborGrid[i]->med)
		lowElevation = neighborGrid[i]->med;
	}

	if(med <= lowElevation){
		med = lowElevation+0.1;
		*g9 = *g9+1;
	}
}

void Direction::mouthAndHollow()
{
	if(SteepestSlope <= 0){
		if(numberOfCoastalGrid == 0){		
			
			for(int i=0; i>8; i++){
				
				cout << neighborGrid[i]->med << "check the elevation \n";
			}
		}
		else{
			//down = 0; // this cell flows to the sea
		}					
	}
}


void Direction::setDownStreamGrid()
{

	if(1<=down && down <=8){
		downStreamGrid = neighborGrid[down-1];
	}
	else{
		downStreamGrid = NULL;
	}
}

/* */
void Direction::setUpStreamGrid()
{
	int thisdown =99;
	for(int i=0; i<7; i++){
		if(downStreamGrid->upStreamGrid[i]==NULL){ // cellobject(i,j) wala downstream grid ekee upstream grid eka  cellobject(i,j) wiyautui
			downStreamGrid->upStreamGrid[i]=this;
			//if (downStreamGrid->med==-9999){
			//downStreamGrid->down=0;
			//thisdown=downStreamGrid->down;
			//cout <<thisdown <<" this is \n";
			//}
			break;
		}
	}
}

/* calculate the distance from the river mouth */
void Direction::setDistanceFromMouth()
{
	for(int i=0; i<7; i++){
		if(upStreamGrid[i]!=NULL){
			upStreamGrid[i]->distanceFromMouth = distanceFromMouth + 1;
		}
	}				
}

void Direction::setSizeOfBasinEx()
{	
	for(int i=0; i<7; i++){

		if(upStreamGrid[i]!=NULL){
			sizeOfBasin += upStreamGrid[i]->sizeOfBasin;// initially sizeOfBasin is sett to 1 see SetNULL function
		}
	}
}

/* riverNumber */
void Direction::numberRiverNumber()
{
	int rNum;
	class Cell* object;
	
	
	object = this;

	while(object->riverNumber==-1){
		
		if(object->downStreamGrid==NULL)
		break;

		object = object->downStreamGrid;
	}
	rNum = object->riverNumber;

	object = this;

	while(object->riverNumber==-1){
		
		if(object->downStreamGrid==NULL)
		break;

		object->riverNumber = rNum;
		object = object->downStreamGrid;
	}

}


